@@ -1,5 +1,5 @@
 /*
- * built-in program copyright (C) 2009 - 2011 H.Niwa
+ * built-in program copyright (C) 2009 - 2012 H.Niwa
  */
 
 
@@ -91,9 +91,6 @@ extern jmp_buf program_jb;
 
 int incflag = 0;
 
-extern void PushStack(Context* cx, Node* goals, Node* md, Node* env);
-extern int PopStack(Context* cx, Node* &goals, Node* &md, Node* &env);
-
 extern FILE* MksTemp(char* templ);
 
 extern int BreakFlag;
@@ -108,6 +105,9 @@ int Nop(Context* cx, Node* goalscar, List* module);
 int IncludeSub(Context* cx, Node* goalscar, List* module, char* libname);
 int Include(Context* cx, Node* goalscar, List* module);
 int Load(Context* cx, Node* goalscar, List* module);
+int LoadPred(Context* cx, Node* goalscar, List* module);
+int LoadPredA(Context* cx, Node* goalscar, List* module);
+int CallPred(Context* cx, Node* goalscar, List* module);
 int Save(Context* cx, Node* goalscar, List* module);
 int Editor(Context* cx, Node* goalscar, List* module);
 int ChDir(Context* cx, Node* goalscar, List* module);
@@ -129,6 +129,7 @@ int Warn(Context* cx, Node* goalscar, List* module);
 
 int GetModule(Context* cx, Node* goalscar, List* module);
 int DoUnify(Context* cx, Node* goalscar, List* module);
+int DoInherit(Context* cx, Node* goalscar, List* module);
 int Catch(Context* cx, Node* goalscar, List* module);
 int Catch_SIGINT(Context* cx, Node* goalscar, List* module);
 int EventThrow(Node* ev);
@@ -169,12 +170,16 @@ int Super(Context* cx, Node* goalscar, List* module);
 
 int DoSetVar(Context* cx, Node* goalscar, List* module);
 int DoSetArray(Context* cx, Node* goalscar, List* module);
+int DoGetVar(Context* cx, Node* goalscar, List* module);
+int DoGetArray(Context* cx, Node* goalscar, List* module);
 int DoDelVar(Context* cx, Node* goalscar, List* module);
 int DoDelArray(Context* cx, Node* goalscar, List* module);
 
-int ObjSetVar(Node* nobj, Node* nvar, Node* val, List* module);
+int ObjSetVar(Context* cx, Node* nobj, Node* nvar, Node* val, List* module);
+int ObjSetArray(Context* cx, Node* nobj, Node* nvar, Node* val, Node* index, List* module);
+int ObjGetVar(Context* cx, Node* nobj, Node* nvar, Node* nname, List* module);
+int ObjGetArray(Context* cx, Node* nobj, Node* nvar, Node* nname, Node* index, List* module);
 int ObjDelVar(Node* nobj, Node* nvar, List* module);
-int ObjSetArray(Node* nobj, Node* nvar, Node* val, Node* index, List* module);
 int ObjDelArray(Node* nobj, Node* nvar, Node* index, List* module);
 
 extern int dois(Context* cx, Node* n, List* module);
@@ -427,6 +432,15 @@ int builtin(Context* cx, Node* goalscar, Node* &goalscdr,
 		} else if (s == "load") {
 			r = Load(cx, goalscar, module);
 			return 1;
+		} else if (s == "loadpred") {
+			r = LoadPred(cx, goalscar, module);
+			return 1;
+		} else if (s == "loadpreda") {
+			r = LoadPredA(cx, goalscar, module);
+			return 1;
+		} else if (s == "callpred") {
+			r = CallPred(cx, goalscar, module);
+			return 1;
 		} else if (s == "save") {
 			r = Save(cx, goalscar, module);
 			return 1;
@@ -460,6 +474,9 @@ int builtin(Context* cx, Node* goalscar, Node* &goalscdr,
 		} else if (s == "unify") {
 			r = DoUnify(cx, goalscar, module);
 			return 1;
+		} else if (s == "inherit") {
+			r = DoInherit(cx, goalscar, module);
+			return 1;
 		} else if (s == "catch") {
 			r = Catch(cx, goalscar, module);
 			return 1;
@@ -680,6 +697,12 @@ int builtin(Context* cx, Node* goalscar, Node* &goalscdr,
 		} else if (s == "setArray") {
 			r = DoSetArray(cx, goalscar, module);
 			return 1;
+		} else if (s == "getVar") {
+			r = DoGetVar(cx, goalscar, module);
+			return 1;
+		} else if (s == "getArray") {
+			r = DoGetArray(cx, goalscar, module);
+			return 1;
 		} else if (s == "delVar") {
 			r = DoDelVar(cx, goalscar, module);
 			return 1;
@@ -753,11 +776,15 @@ int xex(Context* cx, Node* goalscar, List* module)
 	}
 	
 	Context* cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->inherit = cx->inherit;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, goalscar);
 	cxpush(cx2, module);
 	int rn=Unify(cx2, goalscar, module);
@@ -898,6 +925,208 @@ int Include(Context* cx, Node* goalscar, List* module)
 	return IncludeSub(cx, goalscar, module, (char*)sfile.c_str());
 }
 
+int LoadPred(Context* cx, Node* goalscar, List* module)
+{
+	Node* g = goalscar->Cdr();
+	if (ListLength(g) < 1) {
+		syserr("usage : <loadpred PREDLIST ...> \n");
+		return 0;
+	}
+
+	Node* npred = g->Val();
+	int rn;
+
+	if (npred->kind() == PRED) {
+		npred = MkList(npred);
+	} else if (npred->kind() != LIST) {
+		syserr("loadpred : PRED argument of args should be a PRED or a LIST. \n");
+		return 0;
+	}
+
+	static char tmpfilename[MAXPATHLEN];
+	strncpy(tmpfilename, tmppath, MAXPATHLEN);
+	strcat(tmpfilename, "/descXXXXXX");
+	FILE* fd = MksTemp(tmpfilename);
+	if (fd == NULL) {
+		printf("tmpfile : cannot open tmp file \n");
+		return 0;
+	}
+
+	npred->printcdr(fd);
+	fprintf(fd, ";");
+	rewind(fd);
+	
+	extern FILE* RdFp;
+	FILE* fdsave = RdFp;
+	RdFp = fd;
+
+	cxpush(cx, Module);
+
+	jmp_buf savejb;
+	memcpy(&savejb, &program_jb, sizeof(jmp_buf));
+	
+	extern  int     NwccMain();
+	int saveincflag = incflag;
+	incflag = 1;
+	List* saveModule = Module;
+	Module = (List*)(Nil->Cons(Nil));
+	
+	NwccMain();
+
+	incflag = saveincflag;
+
+	Node* newModule = Module->Car()->Car();
+	Module = saveModule;
+	Assert(Module, newModule);
+
+	memcpy(&program_jb, &savejb, sizeof(jmp_buf));
+
+	cxpop(cx);
+
+	fclose(fd);
+	RdFp = fdsave;
+
+	unlink(tmpfilename);
+
+	incflag = 1;
+	return 1;
+	
+}
+
+int LoadPredA(Context* cx, Node* goalscar, List* module)
+{
+	Node* g = goalscar->Cdr();
+	if (ListLength(g) < 1) {
+		syserr("usage : <loadpreda PREDLIST ...> \n");
+		return 0;
+	}
+
+	Node* npred = g->Val();
+	int rn;
+
+	if (npred->kind() == PRED) {
+		npred = MkList(npred);
+	} else if (npred->kind() != LIST) {
+		syserr("loadpreda : PRED argument of args should be a PRED or a LIST. \n");
+		return 0;
+	}
+
+	static char tmpfilename[MAXPATHLEN];
+	strncpy(tmpfilename, tmppath, MAXPATHLEN);
+	strcat(tmpfilename, "/descXXXXXX");
+	FILE* fd = MksTemp(tmpfilename);
+	if (fd == NULL) {
+		printf("tmpfile : cannot open tmp file \n");
+		return 0;
+	}
+
+	npred->printcdr(fd);
+	fprintf(fd, ";");
+	rewind(fd);
+	
+	extern FILE* RdFp;
+	FILE* fdsave = RdFp;
+	RdFp = fd;
+
+	cxpush(cx, Module);
+
+	jmp_buf savejb;
+	memcpy(&savejb, &program_jb, sizeof(jmp_buf));
+	
+	extern  int     NwccMain();
+	int saveincflag = incflag;
+	incflag = 1;
+	List* saveModule = Module;
+	Module = (List*)(Nil->Cons(Nil));
+	
+	NwccMain();
+
+	incflag = saveincflag;
+
+	Node* newModule = Module->Car()->Car();
+	Module = saveModule;
+	Asserta(Module, newModule);
+
+	memcpy(&program_jb, &savejb, sizeof(jmp_buf));
+
+	cxpop(cx);
+
+	fclose(fd);
+	RdFp = fdsave;
+
+	unlink(tmpfilename);
+
+	incflag = 1;
+	return 1;
+	
+}
+
+int CallPred(Context* cx, Node* goalscar, List* module)
+{
+	Node* g = goalscar->Cdr();
+	if (ListLength(g) < 1) {
+		syserr("usage : <callpred PREDLIST> \n");
+		return 0;
+	}
+
+	Node* npred = g->Val();
+	int rn = 0;
+
+	if (npred->kind() == PRED) {
+		npred = MkList(npred);
+	} else if (npred->kind() != LIST) {
+		syserr("callpred : PRED argument of args should be a PRED or a LIST. \n");
+		return 0;
+	}
+
+	static char tmpfilename[MAXPATHLEN];
+	strncpy(tmpfilename, tmppath, MAXPATHLEN);
+	strcat(tmpfilename, "/descXXXXXX");
+	FILE* fd = MksTemp(tmpfilename);
+	if (fd == NULL) {
+		printf("tmpfile : cannot open tmp file \n");
+		return 0;
+	}
+
+	fprintf(fd, "?");
+	npred->printcdr(fd);
+	fprintf(fd, ";");
+	rewind(fd);
+	
+	extern FILE* RdFp;
+	FILE* fdsave = RdFp;
+	RdFp = fd;
+
+	cxpush(cx, Module);
+
+	jmp_buf savejb;
+	memcpy(&savejb, &program_jb, sizeof(jmp_buf));
+
+	extern int resultflg;
+	int saveincflag = incflag;
+	int saveresultflg = resultflg;
+	resultflg = 0;
+	
+	extern  int     NwccMain();	
+	NwccMain();
+
+	incflag = saveincflag;
+	resultflg = saveresultflg;
+	
+	memcpy(&program_jb, &savejb, sizeof(jmp_buf));
+
+	cxpop(cx);
+
+	fclose(fd);
+	RdFp = fdsave;
+
+	unlink(tmpfilename);
+
+	incflag = 1;
+	return 1;
+	
+}
+
 int Load(Context* cx, Node* goalscar, List* module)
 {
 	Node* g = goalscar->Cdr();
@@ -1400,7 +1629,7 @@ int Quote(Context* cx, Node* goalscar, List* module)
 int EscSeq(Context* cx, Node* goalscar, List* module)
 {
 	if (ListLength(goalscar->Cdr()) != 2) {
-		syserr("usage : <\\CHAR> \n");
+		syserr("usage : <\\_ CHAR> \n");
 		return 0;
 	}
 
@@ -1620,8 +1849,8 @@ int getvar(Node* var, Node* mod, Node*& val)
 	return 0;
 }
 
-// <Unify module goals>
-int DoUnify(Context* cx, Node* goalscar, List* module)
+static int DoUnifyCommon(Context* cx, Node* goalscar, List* module, 
+							int inherit_flag)
 {
 	int	rn;
 	
@@ -1634,22 +1863,34 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 	Node*	marg = n->Car()->Val();
 	Node*	mname = mka("WORLD");
 
+	std::string new_selfname = "", selfname = cx->selfname;
+	
 	if (marg->Car()->kind() == ATOM) {
+
 		marg = MkPred(marg);
+
+		((Atom*)(marg->Car()))->toString(new_selfname);
 		
-		if (((Atom*)marg->Car())->EqStr("self")) {
-			marg = cx->modulename;
+		if (new_selfname == "self") {
+			new_selfname = cx->selfname;
+		}
+		
+		if (!inherit_flag) {
+			selfname = new_selfname;
 		}
 
-		if (cx->modulename->Eq(mka("WORLD"))) {
-			mname = marg;
-		} else {
-			mname = cx->modulename;
-		}
-		
+		mname = mka(new_selfname);
+		marg = MkPred(mname);
+
 		std::string argname;
 		((Atom*)marg->Car())->toString(argname);
 
+		if (TraceFlag) {
+			printf("::%s ", selfname.c_str());
+			printf("(%s) ", argname.c_str());
+			PrintNode(n->Cdr()->Car());
+		}
+
 #if 1
 		// sys module pred
 		if (((Atom*)marg->Car())->EqStr("sys")) {
@@ -1690,11 +1931,11 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 				((Atom*)g->Car()->Car())->toString(s);
 				
 				if (s == "setArray") {
-					Node* nobj = marg;
+					Node* nobj = MkPred(mka(selfname));
 					Node* nvar = g->Car()->Cdr()->Car();
 					Node* val = g->Car()->Cdr()->Cdr()->Car();
 					Node* index = g->Car()->Cdr()->Cdr()->Cdr()->Car();
-					int r = ObjSetArray(nobj, nvar, val, index, module);
+					int r = ObjSetArray(cx, nobj, nvar, val, index, module);
 					if (TraceFlag) {
 						if (r > 0) {
 							nobj->Car()->print();
@@ -1708,6 +1949,25 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 						}
 					}
 					return r;
+				} if (s == "getArray") {
+					Node* nobj = MkPred(mka(selfname));
+					Node* nvar = g->Car()->Cdr()->Car();
+					Node* nname = g->Car()->Cdr()->Cdr()->Car();
+					Node* index = g->Car()->Cdr()->Cdr()->Cdr()->Car();
+					int r = ObjGetArray(cx, nobj, nvar, nname, index, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <getArray ");
+							nvar->print();
+							printf(" ");
+							nname->print();
+							printf(" ");
+							index->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				}
 			}
 		}
@@ -1719,10 +1979,10 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 				((Atom*)g->Car()->Car())->toString(s);
 				
 				if (s == "setVar") {
-					Node* nobj = marg;
+					Node* nobj = MkPred(mka(selfname));
 					Node* nvar = g->Car()->Cdr()->Car();
 					Node* val  = g->Car()->Cdr()->Cdr()->Car();
-					int r = ObjSetVar(nobj, nvar, val, module);
+					int r = ObjSetVar(cx, nobj, nvar, val, module);
 					if (TraceFlag) {
 						if (r > 0) {
 							nobj->Car()->print();
@@ -1734,6 +1994,22 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 						}
 					}
 					return r;
+				} if (s == "getVar") {
+					Node* nobj = MkPred(mka(selfname));
+					Node* nvar = g->Car()->Cdr()->Car();
+					Node* nname  = g->Car()->Cdr()->Cdr()->Car();
+					int r = ObjGetVar(cx, nobj, nvar, nname, module);
+					if (TraceFlag) {
+						if (r > 0) {
+							nobj->Car()->print();
+							printf(": <getVar ");
+							nvar->print();
+							printf(" ");
+							nname->print();
+							printf(">...success\n");
+						}
+					}
+					return r;
 				} else if (s == "delArray") {
 					Node* nobj = marg;
 					Node* nvar  = g->Car()->Cdr()->Car();
@@ -1881,13 +2157,18 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 		}
 
 
-		if (!getvar(marg, module->Car(), marg)) {
+		if ((argname != "WORLD") &&
+				(!getvar(marg, module->Car(), marg))) {
 			syserr("obj/unify : cannot find obj module : %s \n", 
 					argname.c_str());
 			return 0;
 		}
 
-		marg = Append(Dup(marg), module->Car());
+		if (argname == "WORLD") {
+			marg = module->Car();
+		} else {
+			marg = Append(Dup(marg), module->Car());
+		}
 		marg = MkList(marg);
 	}
 
@@ -1906,12 +2187,17 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 	}
 
 	Context *cx2 = new Context(m, mname);
+
+	cx2->selfname = selfname;
 	cx2->inherit = cx->inherit;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 	cx2->token = cx->token;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, gl);
 	cxpush(cx2, goalscar);
 	cxpush(cx2, m);
@@ -1947,6 +2233,18 @@ int DoUnify(Context* cx, Node* goalscar, List* module)
 	return 0;
 }
 
+// <Unify module goals>
+int DoUnify(Context* cx, Node* goalscar, List* module)
+{
+	DoUnifyCommon(cx, goalscar, module, 0);
+}
+
+int DoInherit(Context* cx, Node* goalscar, List* module)
+{
+	DoUnifyCommon(cx, goalscar, module, 1);
+}
+
+
 static Node* nthrow_val = Nil;
 
 int Catch(Context* cx, Node* goalscar, List* module)
@@ -1970,11 +2268,15 @@ int Catch(Context* cx, Node* goalscar, List* module)
 	ThrowFlag = 0;
 
 	Context* cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->inherit = cx->inherit;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, goalscar);
 	cxpush(cx2, predarg);
 	cxpush(cx2, module);
@@ -2036,11 +2338,15 @@ int Catch_SIGINT(Context* cx, Node* goalscar, List* module)
 	ThrowFlag = 0;
 
 	Context* cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->inherit = cx->inherit;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, goalscar);
 	cxpush(cx2, predarg);
 	cxpush(cx2, module);
@@ -2142,11 +2448,13 @@ int TimeOut(Context* cx, Node* goalscar, List* module)
 	}
 	
 	Node*	predarg = goalscar->Cdr()->Cdr()->Car();
+/*
 	if (predarg->kind() != PRED) {
 		syserr("usage : <timeout TIMEOUT(usec) *PRED*> \n");
 		return 0;
 	}
 
+*/
 
 	timeout* tim = RegistTime(tm);
 	if (setjmp(tim->jbuf)) {
@@ -2158,11 +2466,15 @@ int TimeOut(Context* cx, Node* goalscar, List* module)
 	}
 
 	Context* cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->inherit = cx->inherit;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, goalscar);
 	cxpush(cx2, predarg);
 	cxpush(cx2, module);
@@ -2207,11 +2519,15 @@ int DoLoop(Context* cx, Node* goalscar, List* module)
 	}
 
 	Context *cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 	cx2->token = cx->token;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, gl);
 	cxpush(cx2, nresult);
 
@@ -2277,11 +2593,15 @@ int DoOrPred(Context* cx, Node* goalscar, List* module)
 		
 		gl = goalscar->Car();
 		cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		cxpush(cx2, gl);
 
 		r = Unify(cx2, gl, module);
@@ -2310,11 +2630,15 @@ int DoAlt(Context* cx, Node* goalscar, List* module)
 	Node*	gl = MkList(goalscar->Cdr());
 
 	Context *cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 	cx2->token = cx->token;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, gl);
 
 	if ((rn=Unify(cx2, gl, module))>0) {
@@ -2335,11 +2659,15 @@ int DoNot(Context* cx, Node* goalscar, List* module)
 	int	r;
 	
 	Context *cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 	cx2->token = cx->token;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, gl);
 
 	r = Unify(cx2, gl, module);
@@ -2350,8 +2678,6 @@ int DoNot(Context* cx, Node* goalscar, List* module)
 	cx2 = 0;
 
 	if (r == 1) {
-		return 0;
-	} else if (r == -1) {
 		return -1;
 	} else {
 		return 1;
@@ -2451,11 +2777,15 @@ int DoFor(Context* cx, Node* goalscar, List* module)
 	Context *cx2;
 	for (i = init_val; i <= to_val; i++) {
 		cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(mka(i));
@@ -2584,11 +2914,15 @@ int DoForeach(Context* cx, Node* goalscar, List* module)
 
 	for ( ; nlist->kind() != ATOM; nlist=nlist->Cdr()) {
 		Context *cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		Node* env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(nlist->Car());
@@ -2755,11 +3089,15 @@ int DoFirstFor(Context* cx, Node* goalscar, List* module)
 	Context *cx2;
 	for (i = init_val; i <= to_val; i++) {
 		cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(mka(i));
@@ -2883,11 +3221,15 @@ int DoFirstForeach(Context* cx, Node* goalscar, List* module)
 
 	for ( ; nlist->kind() != ATOM; nlist=nlist->Cdr()) {
 		Context *cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		Node* env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(nlist->Car());
@@ -2955,7 +3297,6 @@ int DoFirstForeach(Context* cx, Node* goalscar, List* module)
 	return 1;
 }
 
-
 int DoFindAll(Context* cx, Node* goalscar, List* module)
 {
 	int i;
@@ -2970,11 +3311,15 @@ int DoFindAll(Context* cx, Node* goalscar, List* module)
 	gl = Append(gl, MkList(MkPred(mka("unknown"))));
 
 	Context *cx2 = new Context(module, cx->modulename);
+	cx2->selfname = cx->selfname;
 	cx2->ioin = cx->ioin;
 	cx2->ioout = cx->ioout;
 	cx2->tokenflag = cx->tokenflag;
 	cx2->token = cx->token;
 
+	cx2->ode = cx->ode;
+	cx2->integral = cx->integral;
+
 	cxpush(cx2, gl);
 	cxpush(cx2, goalscar);
 				
@@ -3264,8 +3609,11 @@ int Self(Context* cx, Node* goalscar, List* module)
 
 	Node* env = Nil->Cons(Nil);
 
+//printf("selfname %s\n", cx->selfname.c_str());
+//PrintNode("modulename ", cx->modulename);
+
 	SetEnv(env, nvar);
-	((Undef*)(nvar->Val()))->Set(cx->modulename->Car());
+	((Undef*)(nvar->Val()))->Set(mka(cx->selfname));
 
 	PushStack(cx, Nil, Nil, env);
 
@@ -3539,7 +3887,7 @@ int DoSetVar(Context* cx, Node* goalscar, List* module)
 {
 	Node* g = goalscar->Cdr();
 	int ll = ListLength(g);
-	if (ll < 2) {
+	if (ll != 2) {
 		syserr("usage : <setVar NAME VAL> \n");
 		return 0;
 	}
@@ -3566,11 +3914,59 @@ int DoSetVar(Context* cx, Node* goalscar, List* module)
 			return 1;
 		}
 	}
-	Asserta(module, MkList(MkPred(Cons(nvar, MkList(val)))));
+	Asserta(Module, MkList(MkPred(Cons(nvar, MkList(val)))));
 
 	return 1;
 }
 
+int DoGetVar(Context* cx, Node* goalscar, List* module)
+{
+	Node* g = goalscar->Cdr();
+	int ll = ListLength(g);
+	if (ll != 2) {
+		syserr("usage : <getVar VAR VAR_NAME > \n");
+		return 0;
+	}
+	Node* nvar = g->Car()->Val();
+	int rn;
+
+	if ((rn = FuncArg(cx, nvar, goalscar, module)) <= 0) {
+		syserr("getVar: failed in the evaluation of the argument. \n");
+		return 0;
+	}
+
+	if (nvar->kind() != UNDEF) {
+		syserr("usage : <getVar VAR VAR_NAME > \n");
+		return 0;
+	}
+
+	Node* nname = g->Cdr()->Car()->Val();
+
+	if ((rn = FuncArg(cx, nname, goalscar, module)) <= 0) {
+		syserr("getVar: failed in the evaluation of the argument. \n");
+		return 0;
+	}
+
+	Node* md = module->Car();
+
+	for ( ; md != Nil; md = md->Cdr()) {
+		if (ModuleCompare(md->Car()->Car()->Car(), nname) == 0) {
+			Node* nd = md->Car()->Car()->Cdr()->Car();
+			
+			Node* env = Nil->Cons(Nil);
+
+			SetEnv(env, nvar);
+			((Undef*)(nvar->Val()))->Set(nd);
+
+			PushStack(cx, Nil, Nil, env);
+
+			return 1;
+		}
+	}
+
+	return -1;
+}
+
 int DoDelVar(Context* cx, Node* goalscar, List* module)
 {
 	Node* g = goalscar->Cdr();
@@ -3614,7 +4010,7 @@ int DoSetArray(Context* cx, Node* goalscar, List* module)
 	Node* g = goalscar->Cdr();
 
 	int ll = ListLength(g);
-	if (ll < 3) {
+	if (ll != 3) {
 		syserr("usage : <setArray NAME VAL INDEX> \n");
 		return 0;
 	}
@@ -3651,11 +4047,69 @@ int DoSetArray(Context* cx, Node* goalscar, List* module)
 			}
 		}
 	}
-	Asserta(module, MkList(MkPred(Cons(nvar, Cons(val, index)))));
+	Asserta(Module, MkList(MkPred(Cons(nvar, Cons(val, index)))));
 
 	return 1;
 }
 
+int DoGetArray(Context* cx, Node* goalscar, List* module)
+{
+	Node* g = goalscar->Cdr();
+
+	int ll = ListLength(g);
+	if (ll != 3) {
+		syserr("usage : <getArray VAR VAR_NAME INDEX> \n");
+		return 0;
+	}
+	Node* nvar  = g->Car()->Val();
+	int rn;
+
+	if ((rn = FuncArg(cx, nvar, goalscar, module)) <= 0) {
+		syserr("getArray: failed in the evaluation of the argument. \n");
+		return 0;
+	}
+	
+	if (nvar->kind() != UNDEF) {
+		syserr("usage : <getArray VAR VAR_NAME INDEX> \n");
+		return 0;
+	}
+
+	Node* nname = g->Cdr()->Car()->Val();
+
+	if ((rn = FuncArg(cx, nname, goalscar, module)) <= 0) {
+		syserr("getArray: failed in the evaluation of the argument. \n");
+		return 0;
+	}
+
+	Node* index = g->Cdr()->Cdr()->Val();
+
+	if ((rn = FuncArg(cx, index, goalscar, module)) <= 0) {
+		syserr("setArray: failed in the evaluation of the argument. \n");
+		return 0;
+	}
+
+	Node* md = module->Car();
+
+	for ( ; md != Nil; md = md->Cdr()) {
+		if (ModuleCompare(md->Car()->Car()->Car()->Cdr(), nvar) == 0) {
+			if (md->Car()->Car()->Cdr()->Cdr()->Eq(index)) {
+				Node* nd = md->Car()->Car()->Cdr()->Car();
+			
+				Node* env = Nil->Cons(Nil);
+
+				SetEnv(env, nvar);
+				((Undef*)(nvar->Val()))->Set(nd);
+
+				PushStack(cx, Nil, Nil, env);
+				
+				return 1;
+			}
+		}
+	}
+
+	return -1;
+}
+
 int DoDelArray(Context* cx, Node* goalscar, List* module)
 {
 	int flg = 0;
@@ -3714,9 +4168,7 @@ int DoDelArray(Context* cx, Node* goalscar, List* module)
 }
 
 
-
-
-int ObjSetVar(Node* nobj, Node* nvar, Node* val, List* module)
+int ObjSetVar(Context* cx, Node* nobj, Node* nvar, Node* val, List* module)
 {
 	Node* md   = module->Car(); 
 
@@ -3758,7 +4210,91 @@ int ObjSetVar(Node* nobj, Node* nvar, Node* val, List* module)
 		}
 	}
 
-	return 0;
+	return -1;
+}
+
+static Node* get_inherit_module(Node* mname, List* module)
+{
+	Node* md   = module->Car(); 
+
+	mname = MkPred(mname);
+	for ( ; md->kind() != ATOM; md = md->Cdr()) {
+		if (ModuleCompare(md->Car()->Car()->Car(), mname) == 0) {
+			Node* n = md->Car()->Car()->Cdr();
+			return n;
+		}
+	}
+	return Nil;
+}
+
+int ObjGetVar(Context* cx, Node* nobj, Node* nvar, Node* nname, List* module)
+{
+	Node* md   = module->Car(); 
+	
+	if (nvar->kind() == ATOM) {
+		std::string s;
+		((Atom*)nvar)->toString(s);
+		if (CheckReserved(s.c_str())) {
+			return 0;
+		}
+	}
+
+	int nest_inherit = 0;
+	for ( ; md->kind() != ATOM; md = md->Cdr()) {
+		if (ModuleCompare(md->Car()->Car()->Car(), nobj) == 0) {
+			Node* n = md->Car()->Car()->Cdr();
+
+			if (ModuleCompare(n->Car()->Car()->Car(), 
+							nname) == 0) {
+				Node* nd = n->Car()->Car()->Cdr()->Car();
+
+				Node* env = Nil->Cons(Nil);
+
+				SetEnv(env, nvar);
+				((Undef*)(nvar->Val()))->Set(nd);
+
+				PushStack(cx, Nil, Nil, env);
+
+				return 1;
+			}
+
+			for ( ; n->kind() != ATOM; n = n->Cdr()) {
+				if (n->Car()->Car()->Eq(mka("inherit"))) {
+					Node* ninherit = 
+						get_inherit_module(
+							n->Car()->Cdr()->Car(),
+							module);
+					ninherit = Append_rc(ninherit, n->Cdr());
+					n = Cons(Nil, ninherit); // dummy for n = n->Cdr()
+
+					nest_inherit++;
+					if (nest_inherit > 10) {
+						syserr("::obj <getVar> inherit overlaps \n"); 
+						return 0;
+					}
+
+					continue;
+				}
+
+				if (ModuleCompare(n->Car()->Car()->Car(), 
+								nname) == 0) {
+					Node* nd = n->Car()->Car()->Cdr()->Car();
+			
+					Node* env = Nil->Cons(Nil);
+
+					SetEnv(env, nvar);
+					((Undef*)(nvar->Val()))->Set(nd);
+
+					PushStack(cx, Nil, Nil, env);
+
+					return 1;
+				}
+			}
+		}
+	}
+
+	return -1;
+
 }
 
 int ObjDelVar(Node* nobj, Node* nvar, List* module)
@@ -3791,7 +4327,7 @@ int ObjDelVar(Node* nobj, Node* nvar, List* module)
 	return 1;
 }
 
-int ObjSetArray(Node* nobj, Node* nvar, Node* val, Node* index, List* module)
+int ObjSetArray(Context* cx, Node* nobj, Node* nvar, Node* val, Node* index, List* module)
 {
 	Node* md   = module->Car(); 
 
@@ -3838,7 +4374,83 @@ int ObjSetArray(Node* nobj, Node* nvar, Node* val, Node* index, List* module)
 		}
 	}
 
-	return 0;
+	return -1;
+}
+
+
+int ObjGetArray(Context* cx, Node* nobj, Node* nvar, Node* nname, Node* index, List* module)
+{
+	Node* md   = module->Car(); 
+
+	if (nvar->kind() == ATOM) {
+		std::string s;
+		((Atom*)nvar)->toString(s);
+		if (CheckReserved(s.c_str())) {
+			return 0;
+		}
+	}
+
+	int nest_inherit = 0;
+	for ( ; md->kind() != ATOM; md = md->Cdr()) {
+		if (ModuleCompare(md->Car()->Car()->Car(), nobj) == 0) {
+			Node* n = md->Car()->Car()->Cdr();
+			if ((ModuleCompare(n->Car()->Car()->Car(), 
+							nname) == 0) 
+			  && (ModuleCompare(n->Car()->Car()->Cdr()->Cdr(), 
+			  				index) == 0)) {
+				Node* nd = n->Car()->Car()->Cdr()->Car();
+
+				Node* env = Nil->Cons(Nil);
+
+				SetEnv(env, nvar);
+				((Undef*)(nvar->Val()))->Set(nd);
+
+				PushStack(cx, Nil, Nil, env);
+
+				return 1;
+			}
+
+			for ( ; n->kind() != ATOM; n = n->Cdr()) {
+
+				if (n->Car()->Car()->Eq(mka("inherit"))) {
+					Node* ninherit = 
+						get_inherit_module(
+							n->Car()->Cdr()->Car(),
+							module);
+					ninherit = Append_rc(ninherit, n->Cdr());
+					n = Cons(Nil, ninherit); // dummy for n = n->Cdr()
+
+					nest_inherit++;
+					if (nest_inherit > 10) {
+						syserr("::obj <getVar> inherit overlaps \n"); 
+						return 0;
+					}
+
+					continue;
+				}
+
+				if ((ModuleCompare(n->Car()->Car()->Car()->Cdr(), 
+								nname) == 0) 
+				  && (ModuleCompare(n->Car()->Car()->Cdr()->Cdr(),
+				  			index) == 0)) {
+					Node* nd = n->Car()->Car()->Cdr()->Car();
+
+					Node* env = Nil->Cons(Nil);
+
+					SetEnv(env, nvar);
+					((Undef*)(nvar->Val()))->Set(nd);
+
+					PushStack(cx, Nil, Nil, env);
+
+					return 1;
+				}
+			}
+
+			return -1;
+		}
+	}
+
+	return -1;
 }
 
 
