@@ -1,5 +1,5 @@
 /*
- * let & rpn calculation H.Niwa copyright (C) 2009 - 2011
+ * let & rpn calculation H.Niwa copyright (C) 2009 - 2012
  */
 
 /*
@@ -265,11 +265,15 @@ int int_intp(Context* cx, Node* n1)
 		Node*	 n1p = n1->Val();
 		Node*	 rval;
 		Context* cx2 = new Context(cx->module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		cxpush(cx, n1p);
 //		if ((rn=Unify(cx2, n1p, cx->module))>0) {
 		if ((rn=FuncPred(cx2, n1p, cx->module, rval))>0) {
@@ -802,11 +806,15 @@ int float_intp(Context* cx, Node* n1)
 		Node* n1p = n1->Val();
 		Node*	 rval;
 		Context* cx2 = new Context(cx->module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
-		
+
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		cxpush(cx, n1p);
 //		if ((rn=Unify(cx2, n1p, cx->module))>0) {
 		if ((rn=FuncPred(cx2, n1p, cx->module, rval))>0) {
@@ -1339,11 +1347,15 @@ int complex_intp(Context* cx, Node* n1)
 		Node* n1p = n1->Val();
 		Node*	 rval;
 		Context* cx2 = new Context(cx->module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
-		
+
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		cxpush(cx, n1p);
 //		if ((rn=Unify(cx2, n1p, cx->module))>0) {
 		if ((rn=FuncPred(cx2, n1p, cx->module, rval))>0) {
