@@ -1,5 +1,5 @@
 /*
- * parallel process program copyright (C) 2009 H.Niwa
+ * parallel process program copyright (C) 2009 - 2012 H.Niwa
  */
 
 
@@ -33,6 +33,7 @@
 #include <libgen.h>
 #include <setjmp.h>
 #include <string.h>
+#include <signal.h>
 
 #ifndef __CYGWIN__
 #ifndef __MINGW32__ 
@@ -90,8 +91,6 @@ extern int DoForeach(Context* cx, Node* goalscar, List* module);
 extern int DoFirstFor(Context* cx, Node* goalscar, List* module);
 extern int DoFirstForeach(Context* cx, Node* goalscar, List* module);
 
-extern void PushStack(Context* cx, Node* goals, Node* md, Node* env);
-extern int PopStack(Context* cx, Node* &goals, Node* &md, Node* &env);
 
 extern FILE* MksTemp(char* templ);
 
@@ -101,7 +100,7 @@ extern FILE* MksTemp(char* templ);
 #if !defined(__MINGW32__) 
 int NewProc(Context* cx, Node* goalscar, List* module)
 {
-	if (ProcFlag) {
+	if (ProcFlag || TraceFlag) {
 		return DoFor(cx, goalscar, module);
 	}
 
@@ -191,7 +190,7 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 	char tmpfilename[MAXPATHLEN];
 	if (nresult != Nil) {
 		strncpy(tmpfilename, tmppath, MAXPATHLEN);
-		strncat(tmpfilename, "/sockXXXXXX", MAXPATHLEN);
+		strncat(tmpfilename, "/sockXXXXXX", MAXPATHLEN-strlen(tmppath)-1);
 		FILE* fd = MksTemp(tmpfilename);
 		if (fd == NULL) {
 			printf("tmpfile : cannot open tmp file \n");
@@ -202,11 +201,15 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 	}
 	
 
+	fflush(cx->ioin);
+	fflush(cx->ioout);
+
 	pid_t spid = fork();
 	if (spid < 0) {
 		syserr("newproc fork error \n");
 		return 0;
-	} if (spid > 0) {
+	} 
+	if (spid > 0) {
 		Node* val = Nil;
 		int pidno = to_val-init_val+1;
 
@@ -244,6 +247,16 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 				ncore++;
 				if (pid == 0) {
 					ProcFlag = 1;
+#ifdef __MINGW32__
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					srand(tv.tv_usec);
+#else
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					srandom(tv.tv_usec);
+#endif /* __MINGW32__ */
+
 				} else if (pid != 0) {
 					savepid[pidno] = pid;
 					int j;
@@ -277,12 +290,16 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 			}
 		}
 
-		cx2 = new Context(module);
+		cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+		
 		env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(mka((long long)i));
@@ -366,7 +383,7 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 
 int EachProc(Context* cx, Node* goalscar, List* module)
 {
-	if (ProcFlag) {
+	if (ProcFlag || TraceFlag) {
 		return DoForeach(cx, goalscar, module);
 	}
 
@@ -417,14 +434,14 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 	glargs = glargs->Cdr();
 
 	Node* nlist = glargs->Car()->Val();
-	if (nlist->kind() != LIST) {
+	if ((nlist->kind() != LIST) && (nlist != Nil)) {
 		return 0;
 	}
 
 	char tmpfilename[MAXPATHLEN];
 	if (nresult != Nil) {
 		strncpy(tmpfilename, tmppath, MAXPATHLEN);
-		strncat(tmpfilename, "/sockXXXXXX", MAXPATHLEN);
+		strncat(tmpfilename, "/sockXXXXXX", MAXPATHLEN-strlen(tmppath)-1);
 		FILE* fd = MksTemp(tmpfilename);
 		if (fd == NULL) {
 			printf("tmpfile : cannot open tmp file \n");
@@ -434,11 +451,15 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 		unlink(tmpfilename);
 	}
 		
+	fflush(cx->ioin);
+	fflush(cx->ioout);
+
 	pid_t spid = fork();
 	if (spid < 0) {
 		syserr("firsteachproc fork error \n");
 		return 0;
-	} if (spid > 0) {
+	} 
+	if (spid > 0) {
 
 		Node* val = Nil;
 		int pidno = ListLength(nlist);
@@ -476,6 +497,15 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 				ncore++;
 				if (pid == 0) {
 					ProcFlag = 1;
+#ifdef __MINGW32__
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					srand(tv.tv_usec);
+#else
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					srandom(tv.tv_usec);
+#endif /* __MINGW32__ */
 				} else if (pid != 0) {
 					savepid[pidno] = pid;
 					int j;
@@ -510,12 +540,16 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 				
 		}
 
-		Context *cx2 = new Context(module);
+		Context *cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		Node* env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(nlist->Car());
@@ -604,7 +638,7 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 
 int FirstNewProc(Context* cx, Node* goalscar, List* module)
 {
-	if (ProcFlag) {
+	if (ProcFlag || TraceFlag) {
 		return DoFirstFor(cx, goalscar, module);
 	}
 
@@ -694,7 +728,7 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 	char tmpfilename[MAXPATHLEN];
 	if (nresult != Nil) {
 		strncpy(tmpfilename, tmppath, MAXPATHLEN);
-		strncat(tmpfilename, "/sockXXXXXX", MAXPATHLEN);
+		strncat(tmpfilename, "/sockXXXXXX", MAXPATHLEN-strlen(tmppath)-1);
 		FILE* fd = MksTemp(tmpfilename);
 		if (fd == NULL) {
 			printf("tmpfile : cannot open tmp file \n");
@@ -704,11 +738,15 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 		unlink(tmpfilename);
 	}
 	
+	fflush(cx->ioin);
+	fflush(cx->ioout);
+
 	pid_t spid = fork();
 	if (spid < 0) {
 		syserr("firstnewproc fork error \n");
 		return 0;
-	} if (spid > 0) {
+	} 
+	if (spid > 0) {
 
 		Node* val = Nil;
 		int pidno = to_val-init_val+1;
@@ -748,6 +786,15 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 				ncore++;
 				if (pid == 0) {
 					ProcFlag = 1;
+#ifdef __MINGW32__
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					srand(tv.tv_usec);
+#else
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					srandom(tv.tv_usec);
+#endif /* __MINGW32__ */
 				} else if (pid != 0) {
 					savepid[pidno] = pid;
 					int j;
@@ -813,12 +860,16 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 			}
 		}
 	
-		cx2 = new Context(module);
+		cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(mka((long long)i));
@@ -912,7 +963,7 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 
 int FirstEachProc(Context* cx, Node* goalscar, List* module)
 {
-	if (ProcFlag) {
+	if (ProcFlag || TraceFlag) {
 		return DoFirstForeach(cx, goalscar, module);
 	}
 
@@ -964,14 +1015,14 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 	glargs = glargs->Cdr();
 
 	Node* nlist = glargs->Car()->Val();
-	if (nlist->kind() != LIST) {
+	if ((nlist->kind() != LIST) && (nlist != Nil)) {
 		return 0;
 	}
 
 	char tmpfilename[MAXPATHLEN];
 	if (nresult != Nil) {
 		strncpy(tmpfilename, tmppath, MAXPATHLEN);
-		strncat(tmpfilename, "/sockXXXXXX", MAXPATHLEN);
+		strncat(tmpfilename, "/sockXXXXXX", MAXPATHLEN-strlen(tmppath));
 		FILE* fd = MksTemp(tmpfilename);
 		if (fd == NULL) {
 			printf("tmpfile : cannot open tmp file \n");
@@ -981,11 +1032,15 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 		unlink(tmpfilename);
 	}
 	
+	fflush(cx->ioin);
+	fflush(cx->ioout);
+
 	pid_t spid = fork();
 	if (spid < 0) {
 		syserr("firsteachproc fork error \n");
 		return 0;
-	} if (spid > 0) {
+	} 
+	if (spid > 0) {
 
 		Node* val = Nil;
 		int pidno = ListLength(nlist);
@@ -1022,6 +1077,15 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 				ncore++;
 				if (pid == 0) {
 					ProcFlag = 1;
+#ifdef __MINGW32__
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					srand(tv.tv_usec);
+#else
+					struct timeval tv;
+					gettimeofday(&tv, NULL);
+					srandom(tv.tv_usec);
+#endif /* __MINGW32__ */
 				} else if (pid != 0) {
 					savepid[pidno] = pid;
 					int j;
@@ -1087,12 +1151,16 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 			}
 		}
 
-		Context *cx2 = new Context(module);
+		Context *cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		Node* env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(nlist->Car());
@@ -1194,6 +1262,8 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 
 #else  /* !defined(__MINGW32__) */
 
+extern int BreakFlag;
+
 int NewProc(Context* cx, Node* goalscar, List* module)
 {
 	long long i;
@@ -1285,12 +1355,16 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 	Node* env = Nil->Cons(Nil);
 	Context *cx2;
 	for (i = init_val; i <= to_val; i++) {
-		cx2 = new Context(module);
+		cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(mka(i));
@@ -1334,6 +1408,8 @@ int NewProc(Context* cx, Node* goalscar, List* module)
 		delete cx2;
 		cx2 = 0;
 
+		BreakFlag = 0;
+
 		if (r != 1) {
 			return r;
 		}
@@ -1401,7 +1477,7 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 	glargs = glargs->Cdr();
 
 	Node* nlist = glargs->Car()->Val();
-	if (nlist->kind() != LIST) {
+	if ((nlist->kind() != LIST) && (nlist != Nil)) {
 		syserr("usage : <eachproc [VAR] (VAR LIST) PRED> \n");
 		return 0;
 	}
@@ -1409,12 +1485,16 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 	Node* val = Nil;
 
 	for ( ; nlist->kind() != ATOM; nlist=nlist->Cdr()) {
-		Context *cx2 = new Context(module);
+		Context *cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		Node* env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(nlist->Car());
@@ -1461,6 +1541,9 @@ int EachProc(Context* cx, Node* goalscar, List* module)
 		
 		delete cx2;
 		cx2 = 0;
+
+		BreakFlag = 0;
+
 /*
 		if (rn != 1) {
 			return rn;
@@ -1572,12 +1655,16 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 	Node* env = Nil->Cons(Nil);
 	Context *cx2;
 	for (i = init_val; i <= to_val; i++) {
-		cx2 = new Context(module);
+		cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(mka(i));
@@ -1622,6 +1709,8 @@ int FirstNewProc(Context* cx, Node* goalscar, List* module)
 		delete cx2;
 		cx2 = 0;
 
+		BreakFlag = 0;
+
 		if (r == 1) {
 			break;
 		}
@@ -1689,7 +1778,7 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 	glargs = glargs->Cdr();
 
 	Node* nlist = glargs->Car()->Val();
-	if (nlist->kind() != LIST) {
+	if ((nlist->kind() != LIST) && (nlist != Nil)) {
 		syserr("usage : <firsteachproc [VAR] (VAR LIST) PRED> \n");
 		return 0;
 	}
@@ -1697,12 +1786,16 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 	Node* val = Nil;
 
 	for ( ; nlist->kind() != ATOM; nlist=nlist->Cdr()) {
-		Context *cx2 = new Context(module);
+		Context *cx2 = new Context(module, cx->modulename);
+		cx2->selfname = cx->selfname;
 		cx2->ioin = cx->ioin;
 		cx2->ioout = cx->ioout;
 		cx2->tokenflag = cx->tokenflag;
 		cx2->token = cx->token;
 
+		cx2->ode = cx->ode;
+		cx2->integral = cx->integral;
+
 		Node* env = Nil->Cons(Nil);
 		SetEnv(env, nvar);
 		((Undef*)nvar)->Set(nlist->Car());
@@ -1750,6 +1843,8 @@ int FirstEachProc(Context* cx, Node* goalscar, List* module)
 		delete cx2;
 		cx2 = 0;
 
+		BreakFlag = 0;
+
 		if (rn == 1) {
 			break;
 		}
